import numpy

from . import basics
from . import polar
from . import utils
from . import phenometrics
from . import metrics
from . import fractal

# Version of the package

__version__ = "0.1.4"

name = "stmetrics"